// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ffi;
use glib::GStr;

#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_BACKGROUND")]
pub static ACCESSIBLE_ATTRIBUTE_BACKGROUND: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_BACKGROUND) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_FAMILY")]
pub static ACCESSIBLE_ATTRIBUTE_FAMILY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_FAMILY) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_FOREGROUND")]
pub static ACCESSIBLE_ATTRIBUTE_FOREGROUND: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_FOREGROUND) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_OVERLINE")]
pub static ACCESSIBLE_ATTRIBUTE_OVERLINE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_OVERLINE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_OVERLINE_NONE")]
pub static ACCESSIBLE_ATTRIBUTE_OVERLINE_NONE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_OVERLINE_NONE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_OVERLINE_SINGLE")]
pub static ACCESSIBLE_ATTRIBUTE_OVERLINE_SINGLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_OVERLINE_SINGLE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_SIZE")]
pub static ACCESSIBLE_ATTRIBUTE_SIZE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_SIZE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_CONDENSED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_CONDENSED: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_CONDENSED) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_EXPANDED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_EXPANDED: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_EXPANDED) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_EXTRA_CONDENSED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_EXTRA_CONDENSED: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_EXTRA_CONDENSED)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_EXTRA_EXPANDED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_EXTRA_EXPANDED: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_EXTRA_EXPANDED)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_NORMAL")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_NORMAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_NORMAL) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_SEMI_CONDENSED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_SEMI_CONDENSED: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_SEMI_CONDENSED)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_SEMI_EXPANDED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_SEMI_EXPANDED: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_SEMI_EXPANDED)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_ULTRA_CONDENSED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_ULTRA_CONDENSED: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_ULTRA_CONDENSED)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_ULTRA_EXPANDED")]
pub static ACCESSIBLE_ATTRIBUTE_STRETCH_ULTRA_EXPANDED: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRETCH_ULTRA_EXPANDED)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STRIKETHROUGH")]
pub static ACCESSIBLE_ATTRIBUTE_STRIKETHROUGH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STRIKETHROUGH) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STYLE")]
pub static ACCESSIBLE_ATTRIBUTE_STYLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STYLE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STYLE_ITALIC")]
pub static ACCESSIBLE_ATTRIBUTE_STYLE_ITALIC: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STYLE_ITALIC) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STYLE_NORMAL")]
pub static ACCESSIBLE_ATTRIBUTE_STYLE_NORMAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STYLE_NORMAL) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_STYLE_OBLIQUE")]
pub static ACCESSIBLE_ATTRIBUTE_STYLE_OBLIQUE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_STYLE_OBLIQUE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE")]
pub static ACCESSIBLE_ATTRIBUTE_UNDERLINE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_DOUBLE")]
pub static ACCESSIBLE_ATTRIBUTE_UNDERLINE_DOUBLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_DOUBLE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_ERROR")]
pub static ACCESSIBLE_ATTRIBUTE_UNDERLINE_ERROR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_ERROR) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_NONE")]
pub static ACCESSIBLE_ATTRIBUTE_UNDERLINE_NONE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_NONE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_SINGLE")]
pub static ACCESSIBLE_ATTRIBUTE_UNDERLINE_SINGLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_UNDERLINE_SINGLE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_ALL_PETITE_CAPS")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT_ALL_PETITE_CAPS: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_ALL_PETITE_CAPS)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_ALL_SMALL_CAPS")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT_ALL_SMALL_CAPS: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_ALL_SMALL_CAPS)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_PETITE_CAPS")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT_PETITE_CAPS: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_PETITE_CAPS)
};
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_SMALL_CAPS")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT_SMALL_CAPS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_SMALL_CAPS) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_TITLE_CAPS")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT_TITLE_CAPS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_TITLE_CAPS) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_UNICASE")]
pub static ACCESSIBLE_ATTRIBUTE_VARIANT_UNICASE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_VARIANT_UNICASE) };
#[cfg(feature = "v4_14")]
#[cfg_attr(docsrs, doc(cfg(feature = "v4_14")))]
#[doc(alias = "GTK_ACCESSIBLE_ATTRIBUTE_WEIGHT")]
pub static ACCESSIBLE_ATTRIBUTE_WEIGHT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_ACCESSIBLE_ATTRIBUTE_WEIGHT) };
#[doc(alias = "GTK_IM_MODULE_EXTENSION_POINT_NAME")]
pub static IM_MODULE_EXTENSION_POINT_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_IM_MODULE_EXTENSION_POINT_NAME) };
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_FULL")]
pub static LEVEL_BAR_OFFSET_FULL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_LEVEL_BAR_OFFSET_FULL) };
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_HIGH")]
pub static LEVEL_BAR_OFFSET_HIGH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_LEVEL_BAR_OFFSET_HIGH) };
#[doc(alias = "GTK_LEVEL_BAR_OFFSET_LOW")]
pub static LEVEL_BAR_OFFSET_LOW: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_LEVEL_BAR_OFFSET_LOW) };
#[doc(alias = "GTK_MEDIA_FILE_EXTENSION_POINT_NAME")]
pub static MEDIA_FILE_EXTENSION_POINT_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_MEDIA_FILE_EXTENSION_POINT_NAME) };
#[doc(alias = "GTK_PAPER_NAME_A3")]
pub static PAPER_NAME_A3: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_A3) };
#[doc(alias = "GTK_PAPER_NAME_A4")]
pub static PAPER_NAME_A4: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_A4) };
#[doc(alias = "GTK_PAPER_NAME_A5")]
pub static PAPER_NAME_A5: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_A5) };
#[doc(alias = "GTK_PAPER_NAME_B5")]
pub static PAPER_NAME_B5: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_B5) };
#[doc(alias = "GTK_PAPER_NAME_EXECUTIVE")]
pub static PAPER_NAME_EXECUTIVE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_EXECUTIVE) };
#[doc(alias = "GTK_PAPER_NAME_LEGAL")]
pub static PAPER_NAME_LEGAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_LEGAL) };
#[doc(alias = "GTK_PAPER_NAME_LETTER")]
pub static PAPER_NAME_LETTER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PAPER_NAME_LETTER) };
#[doc(alias = "GTK_PRINT_SETTINGS_COLLATE")]
pub static PRINT_SETTINGS_COLLATE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_COLLATE) };
#[doc(alias = "GTK_PRINT_SETTINGS_DEFAULT_SOURCE")]
pub static PRINT_SETTINGS_DEFAULT_SOURCE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_DEFAULT_SOURCE) };
#[doc(alias = "GTK_PRINT_SETTINGS_DITHER")]
pub static PRINT_SETTINGS_DITHER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_DITHER) };
#[doc(alias = "GTK_PRINT_SETTINGS_DUPLEX")]
pub static PRINT_SETTINGS_DUPLEX: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_DUPLEX) };
#[doc(alias = "GTK_PRINT_SETTINGS_FINISHINGS")]
pub static PRINT_SETTINGS_FINISHINGS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_FINISHINGS) };
#[doc(alias = "GTK_PRINT_SETTINGS_MEDIA_TYPE")]
pub static PRINT_SETTINGS_MEDIA_TYPE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_MEDIA_TYPE) };
#[doc(alias = "GTK_PRINT_SETTINGS_NUMBER_UP")]
pub static PRINT_SETTINGS_NUMBER_UP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_NUMBER_UP) };
#[doc(alias = "GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT")]
pub static PRINT_SETTINGS_NUMBER_UP_LAYOUT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_NUMBER_UP_LAYOUT) };
#[doc(alias = "GTK_PRINT_SETTINGS_N_COPIES")]
pub static PRINT_SETTINGS_N_COPIES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_N_COPIES) };
#[doc(alias = "GTK_PRINT_SETTINGS_ORIENTATION")]
pub static PRINT_SETTINGS_ORIENTATION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_ORIENTATION) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_BASENAME")]
pub static PRINT_SETTINGS_OUTPUT_BASENAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_BASENAME) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_BIN")]
pub static PRINT_SETTINGS_OUTPUT_BIN: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_BIN) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_DIR")]
pub static PRINT_SETTINGS_OUTPUT_DIR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_DIR) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT")]
pub static PRINT_SETTINGS_OUTPUT_FILE_FORMAT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_FILE_FORMAT) };
#[doc(alias = "GTK_PRINT_SETTINGS_OUTPUT_URI")]
pub static PRINT_SETTINGS_OUTPUT_URI: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_OUTPUT_URI) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAGE_RANGES")]
pub static PRINT_SETTINGS_PAGE_RANGES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAGE_RANGES) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAGE_SET")]
pub static PRINT_SETTINGS_PAGE_SET: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAGE_SET) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_FORMAT")]
pub static PRINT_SETTINGS_PAPER_FORMAT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAPER_FORMAT) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_HEIGHT")]
pub static PRINT_SETTINGS_PAPER_HEIGHT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAPER_HEIGHT) };
#[doc(alias = "GTK_PRINT_SETTINGS_PAPER_WIDTH")]
pub static PRINT_SETTINGS_PAPER_WIDTH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PAPER_WIDTH) };
#[doc(alias = "GTK_PRINT_SETTINGS_PRINTER")]
pub static PRINT_SETTINGS_PRINTER: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PRINTER) };
#[doc(alias = "GTK_PRINT_SETTINGS_PRINTER_LPI")]
pub static PRINT_SETTINGS_PRINTER_LPI: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PRINTER_LPI) };
#[doc(alias = "GTK_PRINT_SETTINGS_PRINT_PAGES")]
pub static PRINT_SETTINGS_PRINT_PAGES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_PRINT_PAGES) };
#[doc(alias = "GTK_PRINT_SETTINGS_QUALITY")]
pub static PRINT_SETTINGS_QUALITY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_QUALITY) };
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION")]
pub static PRINT_SETTINGS_RESOLUTION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_RESOLUTION) };
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION_X")]
pub static PRINT_SETTINGS_RESOLUTION_X: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_RESOLUTION_X) };
#[doc(alias = "GTK_PRINT_SETTINGS_RESOLUTION_Y")]
pub static PRINT_SETTINGS_RESOLUTION_Y: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_RESOLUTION_Y) };
#[doc(alias = "GTK_PRINT_SETTINGS_REVERSE")]
pub static PRINT_SETTINGS_REVERSE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_REVERSE) };
#[doc(alias = "GTK_PRINT_SETTINGS_SCALE")]
pub static PRINT_SETTINGS_SCALE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_SCALE) };
#[doc(alias = "GTK_PRINT_SETTINGS_USE_COLOR")]
pub static PRINT_SETTINGS_USE_COLOR: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_USE_COLOR) };
#[doc(alias = "GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA")]
pub static PRINT_SETTINGS_WIN32_DRIVER_EXTRA: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_WIN32_DRIVER_EXTRA) };
#[doc(alias = "GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION")]
pub static PRINT_SETTINGS_WIN32_DRIVER_VERSION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::GTK_PRINT_SETTINGS_WIN32_DRIVER_VERSION) };
