// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

pub fn version() -> &'static str {
if cfg!(feature = "v4_18") {
        "4.18"
    } else if cfg!(feature = "v4_16") {
        "4.16"
    } else if cfg!(feature = "v4_14") {
        "4.14"
    } else if cfg!(feature = "v4_12") {
        "4.12"
    } else if cfg!(feature = "v4_10") {
        "4.10"
    } else if cfg!(feature = "v4_8") {
        "4.8"
    } else if cfg!(feature = "v4_6") {
        "4.6"
    } else if cfg!(feature = "v4_4") {
        "4.4"
    } else if cfg!(feature = "v4_2") {
        "4.2"
    } else {
        "4"
    }
}
