// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "PangoAlignment", sizeof(PangoAlignment), alignof(PangoAlignment));
    printf("%s;%zu;%zu\n", "PangoAnalysis", sizeof(PangoAnalysis), alignof(PangoAnalysis));
    printf("%s;%zu;%zu\n", "PangoAttrClass", sizeof(PangoAttrClass), alignof(PangoAttrClass));
    printf("%s;%zu;%zu\n", "PangoAttrColor", sizeof(PangoAttrColor), alignof(PangoAttrColor));
    printf("%s;%zu;%zu\n", "PangoAttrFloat", sizeof(PangoAttrFloat), alignof(PangoAttrFloat));
    printf("%s;%zu;%zu\n", "PangoAttrFontDesc", sizeof(PangoAttrFontDesc), alignof(PangoAttrFontDesc));
    printf("%s;%zu;%zu\n", "PangoAttrFontFeatures", sizeof(PangoAttrFontFeatures), alignof(PangoAttrFontFeatures));
    printf("%s;%zu;%zu\n", "PangoAttrInt", sizeof(PangoAttrInt), alignof(PangoAttrInt));
    printf("%s;%zu;%zu\n", "PangoAttrLanguage", sizeof(PangoAttrLanguage), alignof(PangoAttrLanguage));
    printf("%s;%zu;%zu\n", "PangoAttrShape", sizeof(PangoAttrShape), alignof(PangoAttrShape));
    printf("%s;%zu;%zu\n", "PangoAttrSize", sizeof(PangoAttrSize), alignof(PangoAttrSize));
    printf("%s;%zu;%zu\n", "PangoAttrString", sizeof(PangoAttrString), alignof(PangoAttrString));
    printf("%s;%zu;%zu\n", "PangoAttrType", sizeof(PangoAttrType), alignof(PangoAttrType));
    printf("%s;%zu;%zu\n", "PangoAttribute", sizeof(PangoAttribute), alignof(PangoAttribute));
    printf("%s;%zu;%zu\n", "PangoBaselineShift", sizeof(PangoBaselineShift), alignof(PangoBaselineShift));
    printf("%s;%zu;%zu\n", "PangoBidiType", sizeof(PangoBidiType), alignof(PangoBidiType));
    printf("%s;%zu;%zu\n", "PangoColor", sizeof(PangoColor), alignof(PangoColor));
    printf("%s;%zu;%zu\n", "PangoCoverageLevel", sizeof(PangoCoverageLevel), alignof(PangoCoverageLevel));
    printf("%s;%zu;%zu\n", "PangoDirection", sizeof(PangoDirection), alignof(PangoDirection));
    printf("%s;%zu;%zu\n", "PangoEllipsizeMode", sizeof(PangoEllipsizeMode), alignof(PangoEllipsizeMode));
    printf("%s;%zu;%zu\n", "PangoFont", sizeof(PangoFont), alignof(PangoFont));
    printf("%s;%zu;%zu\n", "PangoFontClass", sizeof(PangoFontClass), alignof(PangoFontClass));
    printf("%s;%zu;%zu\n", "PangoFontFace", sizeof(PangoFontFace), alignof(PangoFontFace));
    printf("%s;%zu;%zu\n", "PangoFontFaceClass", sizeof(PangoFontFaceClass), alignof(PangoFontFaceClass));
    printf("%s;%zu;%zu\n", "PangoFontFamily", sizeof(PangoFontFamily), alignof(PangoFontFamily));
    printf("%s;%zu;%zu\n", "PangoFontFamilyClass", sizeof(PangoFontFamilyClass), alignof(PangoFontFamilyClass));
    printf("%s;%zu;%zu\n", "PangoFontMap", sizeof(PangoFontMap), alignof(PangoFontMap));
    printf("%s;%zu;%zu\n", "PangoFontMapClass", sizeof(PangoFontMapClass), alignof(PangoFontMapClass));
    printf("%s;%zu;%zu\n", "PangoFontMask", sizeof(PangoFontMask), alignof(PangoFontMask));
    printf("%s;%zu;%zu\n", "PangoFontMetrics", sizeof(PangoFontMetrics), alignof(PangoFontMetrics));
    printf("%s;%zu;%zu\n", "PangoFontScale", sizeof(PangoFontScale), alignof(PangoFontScale));
    printf("%s;%zu;%zu\n", "PangoFontset", sizeof(PangoFontset), alignof(PangoFontset));
    printf("%s;%zu;%zu\n", "PangoFontsetClass", sizeof(PangoFontsetClass), alignof(PangoFontsetClass));
    printf("%s;%zu;%zu\n", "PangoGlyph", sizeof(PangoGlyph), alignof(PangoGlyph));
    printf("%s;%zu;%zu\n", "PangoGlyphGeometry", sizeof(PangoGlyphGeometry), alignof(PangoGlyphGeometry));
    printf("%s;%zu;%zu\n", "PangoGlyphInfo", sizeof(PangoGlyphInfo), alignof(PangoGlyphInfo));
    printf("%s;%zu;%zu\n", "PangoGlyphItem", sizeof(PangoGlyphItem), alignof(PangoGlyphItem));
    printf("%s;%zu;%zu\n", "PangoGlyphItemIter", sizeof(PangoGlyphItemIter), alignof(PangoGlyphItemIter));
    printf("%s;%zu;%zu\n", "PangoGlyphString", sizeof(PangoGlyphString), alignof(PangoGlyphString));
    printf("%s;%zu;%zu\n", "PangoGlyphUnit", sizeof(PangoGlyphUnit), alignof(PangoGlyphUnit));
    printf("%s;%zu;%zu\n", "PangoGlyphVisAttr", sizeof(PangoGlyphVisAttr), alignof(PangoGlyphVisAttr));
    printf("%s;%zu;%zu\n", "PangoGravity", sizeof(PangoGravity), alignof(PangoGravity));
    printf("%s;%zu;%zu\n", "PangoGravityHint", sizeof(PangoGravityHint), alignof(PangoGravityHint));
    printf("%s;%zu;%zu\n", "PangoItem", sizeof(PangoItem), alignof(PangoItem));
    printf("%s;%zu;%zu\n", "PangoLayoutDeserializeError", sizeof(PangoLayoutDeserializeError), alignof(PangoLayoutDeserializeError));
    printf("%s;%zu;%zu\n", "PangoLayoutDeserializeFlags", sizeof(PangoLayoutDeserializeFlags), alignof(PangoLayoutDeserializeFlags));
    printf("%s;%zu;%zu\n", "PangoLayoutRun", sizeof(PangoLayoutRun), alignof(PangoLayoutRun));
    printf("%s;%zu;%zu\n", "PangoLayoutSerializeFlags", sizeof(PangoLayoutSerializeFlags), alignof(PangoLayoutSerializeFlags));
    printf("%s;%zu;%zu\n", "PangoMatrix", sizeof(PangoMatrix), alignof(PangoMatrix));
    printf("%s;%zu;%zu\n", "PangoOverline", sizeof(PangoOverline), alignof(PangoOverline));
    printf("%s;%zu;%zu\n", "PangoRectangle", sizeof(PangoRectangle), alignof(PangoRectangle));
    printf("%s;%zu;%zu\n", "PangoRenderPart", sizeof(PangoRenderPart), alignof(PangoRenderPart));
    printf("%s;%zu;%zu\n", "PangoRenderer", sizeof(PangoRenderer), alignof(PangoRenderer));
    printf("%s;%zu;%zu\n", "PangoRendererClass", sizeof(PangoRendererClass), alignof(PangoRendererClass));
    printf("%s;%zu;%zu\n", "PangoScript", sizeof(PangoScript), alignof(PangoScript));
    printf("%s;%zu;%zu\n", "PangoShapeFlags", sizeof(PangoShapeFlags), alignof(PangoShapeFlags));
    printf("%s;%zu;%zu\n", "PangoShowFlags", sizeof(PangoShowFlags), alignof(PangoShowFlags));
    printf("%s;%zu;%zu\n", "PangoStretch", sizeof(PangoStretch), alignof(PangoStretch));
    printf("%s;%zu;%zu\n", "PangoStyle", sizeof(PangoStyle), alignof(PangoStyle));
    printf("%s;%zu;%zu\n", "PangoTabAlign", sizeof(PangoTabAlign), alignof(PangoTabAlign));
    printf("%s;%zu;%zu\n", "PangoTextTransform", sizeof(PangoTextTransform), alignof(PangoTextTransform));
    printf("%s;%zu;%zu\n", "PangoUnderline", sizeof(PangoUnderline), alignof(PangoUnderline));
    printf("%s;%zu;%zu\n", "PangoVariant", sizeof(PangoVariant), alignof(PangoVariant));
    printf("%s;%zu;%zu\n", "PangoWeight", sizeof(PangoWeight), alignof(PangoWeight));
    printf("%s;%zu;%zu\n", "PangoWrapMode", sizeof(PangoWrapMode), alignof(PangoWrapMode));
    return 0;
}
