// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "graphene_box_t", sizeof(graphene_box_t), alignof(graphene_box_t));
    printf("%s;%zu;%zu\n", "graphene_euler_order_t", sizeof(graphene_euler_order_t), alignof(graphene_euler_order_t));
    printf("%s;%zu;%zu\n", "graphene_euler_t", sizeof(graphene_euler_t), alignof(graphene_euler_t));
    printf("%s;%zu;%zu\n", "graphene_frustum_t", sizeof(graphene_frustum_t), alignof(graphene_frustum_t));
    printf("%s;%zu;%zu\n", "graphene_matrix_t", sizeof(graphene_matrix_t), alignof(graphene_matrix_t));
    printf("%s;%zu;%zu\n", "graphene_plane_t", sizeof(graphene_plane_t), alignof(graphene_plane_t));
    printf("%s;%zu;%zu\n", "graphene_point3d_t", sizeof(graphene_point3d_t), alignof(graphene_point3d_t));
    printf("%s;%zu;%zu\n", "graphene_point_t", sizeof(graphene_point_t), alignof(graphene_point_t));
    printf("%s;%zu;%zu\n", "graphene_quad_t", sizeof(graphene_quad_t), alignof(graphene_quad_t));
    printf("%s;%zu;%zu\n", "graphene_quaternion_t", sizeof(graphene_quaternion_t), alignof(graphene_quaternion_t));
    printf("%s;%zu;%zu\n", "graphene_ray_intersection_kind_t", sizeof(graphene_ray_intersection_kind_t), alignof(graphene_ray_intersection_kind_t));
    printf("%s;%zu;%zu\n", "graphene_ray_t", sizeof(graphene_ray_t), alignof(graphene_ray_t));
    printf("%s;%zu;%zu\n", "graphene_rect_t", sizeof(graphene_rect_t), alignof(graphene_rect_t));
    printf("%s;%zu;%zu\n", "graphene_simd4f_t", sizeof(graphene_simd4f_t), alignof(graphene_simd4f_t));
    printf("%s;%zu;%zu\n", "graphene_simd4x4f_t", sizeof(graphene_simd4x4f_t), alignof(graphene_simd4x4f_t));
    printf("%s;%zu;%zu\n", "graphene_size_t", sizeof(graphene_size_t), alignof(graphene_size_t));
    printf("%s;%zu;%zu\n", "graphene_sphere_t", sizeof(graphene_sphere_t), alignof(graphene_sphere_t));
    printf("%s;%zu;%zu\n", "graphene_triangle_t", sizeof(graphene_triangle_t), alignof(graphene_triangle_t));
    printf("%s;%zu;%zu\n", "graphene_vec2_t", sizeof(graphene_vec2_t), alignof(graphene_vec2_t));
    printf("%s;%zu;%zu\n", "graphene_vec3_t", sizeof(graphene_vec3_t), alignof(graphene_vec3_t));
    printf("%s;%zu;%zu\n", "graphene_vec4_t", sizeof(graphene_vec4_t), alignof(graphene_vec4_t));
    return 0;
}
