// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, VolumeMonitor};

glib::wrapper! {
    #[doc(alias = "GNativeVolumeMonitor")]
    pub struct NativeVolumeMonitor(Object<ffi::GNativeVolumeMonitor, ffi::GNativeVolumeMonitorClass>) @extends VolumeMonitor;

    match fn {
        type_ => || ffi::g_native_volume_monitor_get_type(),
    }
}

impl NativeVolumeMonitor {
    pub const NONE: Option<&'static NativeVolumeMonitor> = None;
}
