// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

use crate::ffi;
use glib::{
    prelude::*,
    signal::{connect_raw, SignalHandlerId},
    translate::*,
};
use std::boxed::Box as Box_;

glib::wrapper! {
    #[doc(alias = "AdwLayout")]
    pub struct Layout(Object<ffi::AdwLayout, ffi::AdwLayoutClass>) @implements gtk::Buildable;

    match fn {
        type_ => || ffi::adw_layout_get_type(),
    }
}

impl Layout {
    #[doc(alias = "adw_layout_new")]
    pub fn new(content: &impl IsA<gtk::Widget>) -> Layout {
        assert_initialized_main_thread!();
        unsafe { from_glib_full(ffi::adw_layout_new(content.as_ref().to_glib_none().0)) }
    }

    #[doc(alias = "adw_layout_get_content")]
    #[doc(alias = "get_content")]
    pub fn content(&self) -> gtk::Widget {
        unsafe { from_glib_none(ffi::adw_layout_get_content(self.to_glib_none().0)) }
    }

    #[doc(alias = "adw_layout_get_name")]
    #[doc(alias = "get_name")]
    pub fn name(&self) -> Option<glib::GString> {
        unsafe { from_glib_none(ffi::adw_layout_get_name(self.to_glib_none().0)) }
    }

    #[doc(alias = "adw_layout_set_name")]
    #[doc(alias = "name")]
    pub fn set_name(&self, name: Option<&str>) {
        unsafe {
            ffi::adw_layout_set_name(self.to_glib_none().0, name.to_glib_none().0);
        }
    }

    #[cfg(feature = "v1_6")]
    #[cfg_attr(docsrs, doc(cfg(feature = "v1_6")))]
    #[doc(alias = "name")]
    pub fn connect_name_notify<F: Fn(&Self) + 'static>(&self, f: F) -> SignalHandlerId {
        unsafe extern "C" fn notify_name_trampoline<F: Fn(&Layout) + 'static>(
            this: *mut ffi::AdwLayout,
            _param_spec: glib::ffi::gpointer,
            f: glib::ffi::gpointer,
        ) {
            let f: &F = &*(f as *const F);
            f(&from_glib_borrow(this))
        }
        unsafe {
            let f: Box_<F> = Box_::new(f);
            connect_raw(
                self.as_ptr() as *mut _,
                b"notify::name\0".as_ptr() as *const _,
                Some(std::mem::transmute::<*const (), unsafe extern "C" fn()>(
                    notify_name_trampoline::<F> as *const (),
                )),
                Box_::into_raw(f),
            )
        }
    }
}
